/*global jQuery, Drupal, drupalSettings, window*/
/*jslint white:true, multivar, this, browser:true*/

(function ($, Drupal, drupalSettings, window) {

  "use strict";

  var initialized, threadId, loadingPrev, loadingNew, container, timeout, refreshRate;

  function triggerCommands(data) {
    var ajaxObject = Drupal.ajax({
      url: "",
      base: false,
      element: false,
      progress: false
    });

    // Trigger any any ajax commands in the response
    ajaxObject.success(data, "success");
  }

  function loadPreviousListener(context) {
    $(context).find("#load-previous-messages").once("load-previous-private-messages-listener").each(function() {
      $(this).click(function(e) {
        e.preventDefault();
 
        if (!loadingPrev) {
          loadingPrev = true;

          var oldestId;
          container.find(".private-message").each(function() {
            if (!oldestId || Number($(this).attr("data-message-id")) < oldestId) {
              oldestId = Number($(this).attr("data-message-id"));
            }
          });

          $.ajax({
            url:drupalSettings.privateMessageThreadMessages.previousMessageCheckUrl,
            data: {threadid:threadId, messageid:oldestId},
            success:function(data) {
              loadingPrev = false;
              triggerCommands(data);
            }
          });
        }
      });
    });
  }

  function insertNewMessages(messages) {
    var html = $("<div/>").html(messages).contents().css("display", "none");

    if (drupalSettings.privateMessageThreadMessages.messageOrder === "asc") {
      html.appendTo(container);
    }
    else {
      html.prependTo(container);
    }

    html.slideDown(300);

    Drupal.attachBehaviors();
  }

  function insertPreviousMessages(messages) {
    var html = $("<div/>").html(messages).contents().css("display", "none");

    if (drupalSettings.privateMessageThreadMessages.messageOrder === "asc") {
      html.prependTo(container);
    }
    else {
      html.appendTo(container);
    }

    html.slideDown(300);

    Drupal.attachBehaviors();
  }

  function checkForMessages() {

    if (!loadingNew) {
      var newestId = 0;

      loadingNew = true;

      container.find(".private-message").each(function() {
        if (Number($(this).attr("data-message-id")) > newestId) {
          newestId = Number($(this).attr("data-message-id"));
        }
      });

      $.ajax({
        url:drupalSettings.privateMessageThreadMessages.newMessageCheckUrl,
        data: {threadid:threadId, messageid:newestId},
        success:function(data) {
          triggerCommands(data);

          loadingNew = false;

          // Check for new messages again
          timeout = window.setTimeout(checkForMessages, refreshRate);
        }
      });
    }
  }

  function init() {
    var loadPreviousButton;

    if (!initialized) {
      initialized = true;
      threadId = drupalSettings.privateMessageThreadMessages.threadId;
      refreshRate = drupalSettings.privateMessageThreadMessages.refreshRate;

      container = $(".private-message-thread-full:first .field--name-private-messages:first .field__items:first");
      loadPreviousButton = $("<div/>", {id:"load-previous-messages-button-wrapper"}).append($("<a/>", {href:"#", id:"load-previous-messages"}).text(Drupal.t("Load Previous")));

      if (drupalSettings.privateMessageThreadMessages.messageOrder === "asc") {
        loadPreviousButton.addClass("load-previous-position-before").insertBefore(container);
      }
      else {
        loadPreviousButton.addClass("load-previous-position-after").insertAfter(container);
      }

      timeout = window.setTimeout(checkForMessages, refreshRate);
    }
  }

  Drupal.behaviors.privateMessageThread = {
    attach:function(context) {
      init();
      loadPreviousListener(context);

      Drupal.AjaxCommands.prototype.insertPrivateMessages = function(ajax, response) {
        // stifles jSlint warning.
        ajax = ajax;

        if (response.insertType === "new") {
          insertNewMessages(response.messages);
        }
        else {
          if (response.messages) {
            insertPreviousMessages(response.messages);
          }
          else {
            $("#load-previous-messages").parent().slideUp(300, function() {
              $(this).remove();
            });
          }
        }
      };

      Drupal.AjaxCommands.prototype.loadNewPrivateMessages = function() {

        window.clearTimeout(timeout);

        checkForMessages();
      };
    }
  };

}(jQuery, Drupal, drupalSettings, window));
