<?php

namespace Drupal\private_message\Event;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\private_message\Entity\PrivateMessageThread;
use Drupal\user\Entity\User;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\FinishRequestEvent;

class PrivateMessageEventSubscriber implements EventSubscriberInterface {

  /**
   * The current user
   *
   * @var \Drupal\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Construct a PrivateMessageEventSubscriber object
   *
   * @param \Drupal\Session\AccountProxyInterface $currentUser
   *   The current user
   */
  public function __construct(AccountProxyInterface $currentUser) {
    $this->currentUser = $currentUser;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::FINISH_REQUEST][] = ['onKernelTerminate'];

    return $events;
  }

  /**
   * Register the thread access for the user
   *
   * @param \Symfony\Component\HttpKernel\Event\FinishRequestEvent $event
   */
  public function onKernelTerminate(FinishRequestEvent $event) {
    // Register the thread access
    $private_message_thread_id = $event->getRequest()->get('private_message_thread_id');
    if ($private_message_thread_id) {
      $private_message_thread = PrivateMessageThread::load($private_message_thread_id);
      if ($private_message_thread) {
        $user = User::load($this->currentUser->id());

        $last_access_time = $private_message_thread->getLastAccessTimestamp($user);
        $newest_message_timestamp = $private_message_thread->getNewestMessageCreationTimestamp();

        if ($last_access_time < $newest_message_timestamp) {
          $tags[] = 'private_message_thread:' . $private_message_thread->id() . ':view:uid:' . $user->id();

          Cache::invalidateTags($tags);
        }

        $private_message_thread->updateLastAccessTime($user);
      }
    }
  }
}
